#include "pulse_detector.h"

void detect(pos_t sample_x, data_t sample_y, pos_t& out_x, data_t& out_y, int& ok)
{
    ok = 0;
    out_x = 0.0;
    out_y = 0.0;

    if (status == CALIBRATING)
    {
        if (c_counter < CAL_SIZE)
        {
            average += sample_y;
            c_counter++;
        }
        else
        {
            status = BUFFERING;
            average /= CAL_SIZE;
            c_counter = 0;
        }
    }
    else
    {
        data_t b_sample = sample_y - average;

        // workaround for fabs()
        data_t a_b_sample = b_sample;
        if (a_b_sample < 0.0) a_b_sample = ZERO - a_b_sample;

        if (a_b_sample > a_p_y)
        {
            p_y = b_sample;
            a_p_y = b_sample;
            if (a_p_y < 0.0) a_p_y = ZERO - a_p_y;
        }

        // see https://xilinx.github.io/Vitis_Accel_Examples/2020.1/html/shift_register.html
        SHIFTREG: for (int i = F_SIZE - 1; i > 0 ; i--)
        {
#pragma HLS array_partition variable=buffer complete
#pragma HLS unroll
            buffer[i] = buffer[i - 1];
        }
        buffer[0] = (a_b_sample > THRESHOLD);

        if (status == BUFFERING)
        {
            if (c_counter < F_SIZE)
            {
                c_counter++;
                return;
            }
            else
            {
                status = PULSE_OFF;
            }
        }

        bool active;
        PSUM: for (int i = 0; i < F_SIZE; i++)
        {
#pragma HLS UNROLL
            active |= buffer[i];
        }

        if (active)
        {
            status = PULSE_ON;
        }
        else if (status == PULSE_ON)
        {
            ok = 1;
            out_x = p_x;
            out_y = p_y;

            p_x = 0.0;
            p_y = 0.0;
            a_p_y = 0.0;
            status = PULSE_OFF;
        }
    }
}
